@@ -12,9 +12,9 @@ def has_capabilities(*features):
 
 
 class Helper:
-    def __init__(self, app):
+    def __init__(self, trame_server):
         self._root_protocol = None
-        self._app = app
+        self._trame_server = trame_server
         self._hybrid_views = {}
 
         try:  # defer need to paraview to support --www usecase
@@ -29,7 +29,7 @@ class Helper:
             self._mesh_vtk = mesh_vtk
 
             # Link our custom protocols initialization
-            app.add_protocol_to_configure(self.configure_protocol)
+            trame_server.add_protocol_to_configure(self.configure_protocol)
 
     def id(self, pv_proxy):
         if pv_proxy:
@@ -58,7 +58,9 @@ class Helper:
             cell_arrays=cell_arrays,
         )
 
-    def scene(self, view_proxy, new_state=False):
+    def scene(
+        self, view_proxy, new_state=False, widgets=None, orientation_axis=0, **kwargs
+    ):
         # flush data without requiring a render/picture
         tmp = view_proxy.SuppressRendering
         view_proxy.SuppressRendering = 1
@@ -68,7 +70,19 @@ class Helper:
             view_proxy.SuppressRendering = tmp
 
         return self._app.protocol_call(
-            "viewport.geometry.view.get.state", self.id(view_proxy), new_state
+            "viewport.geometry.view.get.state",
+            self.id(view_proxy),
+            new_state,
+            widgets=widgets,
+            orientation_axis=orientation_axis,
+        )
+
+    def export(self, render_window, widgets=None, orientation_axis=0, **kwargs):
+        return self._trame_server.protocol_call(
+            "viewport.geometry.view.get.export",
+            self.id(render_window),
+            widgets=widgets,
+            orientation_axis=orientation_axis,
         )
 
     def push_image(self, view_proxy, reset_camera=False):
@@ -76,11 +90,11 @@ class Helper:
             view_proxy.EnableRenderOnInteraction = 0
 
         if reset_camera:
-            self._app.protocol_call(
-                "viewport.camera.reset", {"view": self.id(view_proxy)}
+            self._trame_server.protocol_call(
+                "viewport.camera.reset", self.id(view_proxy)
             )
 
-        return self._app.protocol_call(
+        return self._trame_server.protocol_call(
             "viewport.image.push", {"view": self.id(view_proxy)}
         )
 
@@ -180,9 +194,9 @@ class Helper:
 HELPER = None
 
 
-def setup(app, **kwargs):
+def setup(trame_server, **kwargs):
     global HELPER
-    HELPER = Helper(app)
+    HELPER = Helper(trame_server)
 
 
 # -----------------------------------------------------------------------------
@@ -209,6 +223,12 @@ def scene(render_window, reset_camera=False, new_state=True):
     return scene_state
 
 
+def export(render_window, widgets=None, orientation_axis=0):
+    return HELPER.export(
+        render_window, widgets=widgets, orientation_axis=orientation_axis
+    )
+
+
 def push_image(render_window, reset_camera=False):
     return HELPER.push_image(render_window, reset_camera)
 
