@@ -3,7 +3,8 @@ import os
 
 from .mesh import mesh
 from .initialize import initialize_serializers
-from .serialize import serialize
+from .serialize import serialize, serialize_widget
+from .export import extract_array_hash
 from .synchronization_context import SynchronizationContext
 from .utils import reference_id
 
@@ -21,4 +22,6 @@ __all__ = [
     "mesh",
     "serialize",
     "SynchronizationContext",
+    "serialize_widget",
+    "extract_array_hash",
 ]
