@@ -1,6 +1,18 @@
+import io
+import json
+import zipfile
+
 from trame_client.widgets.core import AbstractElement
+
 from trame_vtk.modules import common
 
+try:
+    import zlib  # noqa
+
+    ZIP_COMPRESSION = zipfile.ZIP_DEFLATED
+except ImportError:
+    ZIP_COMPRESSION = zipfile.ZIP_STORED
+
 MODULE = None
 
 
@@ -349,7 +361,7 @@ class VtkRemoteLocalView(HtmlElement):
     ... )
     """
 
-    def __init__(self, view, enable_rendering=True, **kwargs):
+    def __init__(self, view, enable_rendering=True, widgets=[], **kwargs):
         super().__init__("vtk-remote-local-view", **kwargs)
 
         activate_module_for(self.server, view)
@@ -363,6 +375,7 @@ class VtkRemoteLocalView(HtmlElement):
         self.__ref = kwargs.get("ref", __ns)
         self.__rendering = enable_rendering
         self.__namespace = __ns
+        self._widgets = widgets
 
         # !!! HACK !!!
         # Allow user to configure view mode by providing (..., local/remote) and or "local/remote"
@@ -461,17 +474,60 @@ class VtkRemoteLocalView(HtmlElement):
         # print("vtkCommand.EndInteractionEvent", vtkCommand.EndInteractionEvent)
         self.__view.GetInteractor().AddObserver(45, self._push_camera)
 
-    def update_geometry(self, reset_camera=False, **kwargs):
+    def update_geometry(
+        self, reset_camera=False, widgets=None, orientation_axis=0, **kwargs
+    ):
         """
         Force update to geometry
         """
+        if widgets is None:
+            widgets = self._widgets
+
         if self.server.protocol:
-            delta_state = MODULE.scene(self.__view, new_state=False)
+            delta_state = MODULE.scene(
+                self.__view,
+                new_state=False,
+                widgets=widgets,
+                orientation_axis=orientation_axis,
+            )
             self.server.protocol.publish("trame.vtk.delta", delta_state)
 
-        full_state = MODULE.scene(self.__view, new_state=True)
+        full_state = MODULE.scene(
+            self.__view,
+            new_state=True,
+            widgets=widgets,
+            orientation_axis=orientation_axis,
+        )
         self.server.state[self.__scene_id] = full_state
 
+    def export_geometry(self, widgets=None, orientation_axis=0, format="zip", **kwargs):
+        """Export standalone scene for OfflineViewer
+
+        :param format: Can be either be "zip" or "json".
+        """
+        encoded_data = None
+
+        if widgets is None:
+            widgets = self._widgets
+
+        if self.server.protocol:
+            encoded_data = MODULE.export(
+                self.__view,
+                widgets=widgets,
+                orientation_axis=orientation_axis,
+            )
+
+        if encoded_data:
+            json_out = json.dumps(encoded_data)
+            if format == "json":
+                return json_out.encode(encoding="UTF-8", errors="strict")
+
+            zip_buffer = io.BytesIO()
+            with zipfile.ZipFile(zip_buffer, "a") as zfile:
+                zfile.writestr("index.json", json_out, compress_type=ZIP_COMPRESSION)
+
+            return zip_buffer.getvalue()
+
     def update_image(self, reset_camera=False):
         """
         Force update to image
@@ -484,11 +540,15 @@ class VtkRemoteLocalView(HtmlElement):
     def set_remote_rendering(self, remote=True, **kwargs):
         self.server.state[self.__mode_key] = "remote" if remote else "local"
 
-    def update(self, reset_camera=False, **kwargs):
+    def update(self, reset_camera=False, widgets=None, orientation_axis=0, **kwargs):
         # need to do both to keep things in sync
         if self.__rendering:
             self.update_image(reset_camera)
-        self.update_geometry()
+        self.update_geometry(
+            reset_camera=reset_camera,
+            widgets=widgets,
+            orientation_axis=orientation_axis,
+        )
 
         if reset_camera:
             self.server.js_call(
@@ -559,6 +619,13 @@ class VtkRemoteLocalView(HtmlElement):
             opts,
         )
 
+    def get_widgets(self):
+        return self._widgets
+
+    def set_widgets(self, value):
+        self._widgets = value
+        self.update_geometry()
+
 
 class VtkRemoteView(HtmlElement):
     """
@@ -710,7 +777,7 @@ class VtkLocalView(HtmlElement):
     ... )
     """
 
-    def __init__(self, view, ref="view", **kwargs):
+    def __init__(self, view, ref="view", widgets=[], **kwargs):
         super().__init__("vtk-local-view", **kwargs)
 
         activate_module_for(self.server, view)
@@ -722,6 +789,7 @@ class VtkLocalView(HtmlElement):
         self.__ref = ref
         self._attributes["ref"] = f'ref="{ref}"'
         self._attributes["view_state"] = f':viewState="{self.__scene_id}"'
+        self._widgets = widgets
 
         self._attr_names += [
             ("interactor_events", "interactorEvents"),
@@ -778,17 +846,65 @@ class VtkLocalView(HtmlElement):
         self.update()
         self._server.controller.on_server_ready.add(self.update)
 
-    def update(self, **kwargs):
+    def get_widgets(self):
+        return self._widgets
+
+    def set_widgets(self, value):
+        self._widgets = value
+        self.update()
+
+    def update(self, widgets=None, orientation_axis=0, **kwargs):
         """
         Force geometry to be pushed
         """
+        if widgets is None:
+            widgets = self._widgets
+
         if self.server.protocol:
-            delta_state = MODULE.scene(self.__view, new_state=False)
+            delta_state = MODULE.scene(
+                self.__view,
+                new_state=False,
+                widgets=widgets,
+                orientation_axis=orientation_axis,
+            )
             self.server.protocol.publish("trame.vtk.delta", delta_state)
 
-        full_state = MODULE.scene(self.__view, new_state=True)
+        full_state = MODULE.scene(
+            self.__view,
+            new_state=True,
+            widgets=widgets,
+            orientation_axis=orientation_axis,
+        )
         self.server.state[self.__scene_id] = full_state
 
+    def export(self, widgets=None, orientation_axis=0, format="zip", **kwargs):
+        """Export standalone scene for OfflineViewer
+
+        :param format: Can be either be "zip" or "json".
+        """
+        encoded_data = None
+
+        if widgets is None:
+            widgets = self._widgets
+
+        if self.server.protocol:
+            encoded_data = MODULE.export(
+                self.__view,
+                widgets=widgets,
+                orientation_axis=orientation_axis,
+            )
+
+        if encoded_data:
+            json_out = json.dumps(encoded_data)
+            if format == "json":
+                return json_out.encode(encoding="UTF-8", errors="strict")
+
+            zip_buffer = io.BytesIO()
+            with zipfile.ZipFile(zip_buffer, "a") as zfile:
+                zfile.writestr("index.json", json_out, compress_type=ZIP_COMPRESSION)
+
+            return zip_buffer.getvalue()
+
     def reset_camera(self, **kwargs):
         """
         Move camera to center actors within the frame
